import os
from scripts import read_case_from_excel


def write_case_to_rf(rf_path, excel_path):
    case_collect = read_case_from_excel(excel_path)

    valid_cases_sum = 0
    for case_dir in case_collect:
        suite_path = f"{os.path.join(rf_path, case_dir)}.robot"
        suite_dir = os.path.dirname(suite_path)

        if not os.path.exists(suite_dir):
            os.makedirs(suite_dir)

        if os.path.exists(suite_path):
            with open(suite_path, mode="r", encoding="utf8") as f:
                suite_content = f.read()
        else:
            suite_content = "***Test Case***\n"

        for case_name, case_desc in case_collect[case_dir].items():
            if case_name not in suite_content:
                suite_content += f"{case_name}\n{case_desc}\n"
                valid_cases_sum += 1
        with open(suite_path, mode="w", encoding="utf8") as f:
            f.write(suite_content)

    all_cases_sum = sum([len(cases) for cases in case_collect.values()])
    print(f"共添加{valid_cases_sum}个有效用例, 略过{all_cases_sum - valid_cases_sum}个已存在用例")



import os
import xlrd
import json
from operator import eq


def read_case_from_excel(excel_path):
    os.chdir(os.path.dirname(os.path.abspath(__file__)))
    with open("excel_config.json", mode="r", encoding="utf8") as f:
        config = json.load(f)

    wb = xlrd.open_workbook(excel_path)
    if config["caseSheet"] not in wb.sheet_names():
        raise ValueError("找不到excel下对应的sheet页, 请在excel_config.json配置", config["caseSheet"])

    ws = wb.sheet_by_name(config["caseSheet"])
    case_path, case_no, case_title = config["casePath"], config["caseNo"], config["caseTitle"]
    case_prepare, case_step, case_expect = config["casePrep"], config["caseStep"], config["caseExpt"]

    _1st_case_row = 1
    for row in range(ws.nrows):
        if eq(ws.cell_value(row, 0), case_path):
            _1st_case_row = row + 1
    col_name = ws.row_values(_1st_case_row - 1)
    path_col, no_col, title_col, prep_col, step_col, expt_col = col_name.index(case_path), col_name.index(
        case_no), col_name.index(case_title), col_name.index(case_prepare), col_name.index(case_step), col_name.index(
        case_expect)

    cases_collect = {}
    """
    {
        "case_dir1": {
            "case_name1": "case_desc1",
            "case_name2": "case_desc2",
            ...
        },
        ...
    }
    """
    for row in range(_1st_case_row, ws.nrows):
        if ws.cell_value(row, 0) is None:
            continue
        case_dir = ws.cell_value(row, path_col).replace("\\", "/").strip("/")
        case_name = f"{ws.cell_value(row, no_col)}_{ws.cell_value(row, title_col)}"
        case_desc = combine_case_desc(ws.cell_value(row, prep_col), ws.cell_value(row, step_col),
                                      ws.cell_value(row, expt_col))
        if case_dir not in cases_collect.keys():
            cases_collect.update({case_dir: {case_name: case_desc}})
        elif case_name not in cases_collect[case_dir].keys():
            cases_collect[case_dir].update({case_name: case_desc})
    return cases_collect


def combine_case_desc(case_prepare, case_step, case_expect):
    header, gap = "    [Documentation]    ", "    ...    "
    case_prepare = case_prepare.replace('\n', f'\n{gap}')
    case_step = case_step.replace('\n', f'\n{gap}')
    case_expect = case_expect.replace('\n', f'\n{gap}')
    case_desc = f"{header}【前置条件】:\n" \
                f"{gap}{case_prepare}\n" \
                f"{gap}【用例步骤】:\n" \
                f"{gap}{case_step}\n" \
                f"{gap}【预期结果】:\n" \
                f"{gap}{case_expect}\n"
    return case_desc



from setuptools import setup, find_packages

setup(
    name='rf_case_generator',
    version='1.0.1',
    description='auto generate test case of RF with excel',
    license='MIT License',
    install_requires=["xlrd"],
    packages=['rf_case_generator'],
    include_package_data=True,
    author='Patrick Peng',
    author_email='pengjiahao97@gmail.com',
    url='https://github.com/pengjiahao97/robotframework-testcase-generator',
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ]
)
